@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2008-2020 The OpenLDAP Foundation.
+ * Copyright 2008-2022 The OpenLDAP Foundation.
  * Copyright 2008 by Howard Chu, Symas Corp.
  * All rights reserved.
  *
@@ -77,7 +77,7 @@ static int write_ether(nssov_ether_cbp *cbp,Entry *entry)
 		if ( !a )
 		{
 			Debug(LDAP_DEBUG_ANY,"ether entry %s does not contain %s value\n",
-							entry->e_name.bv_val,cbp->mi->mi_attrs[0].an_desc->ad_cname.bv_val,0 );
+							entry->e_name.bv_val,cbp->mi->mi_attrs[0].an_desc->ad_cname.bv_val );
 			return 0;
 		}
 		names = a->a_vals;
@@ -95,7 +95,7 @@ static int write_ether(nssov_ether_cbp *cbp,Entry *entry)
 		if ( !a )
 		{
 			Debug(LDAP_DEBUG_ANY,"ether entry %s does not contain %s value\n",
-							entry->e_name.bv_val,cbp->mi->mi_attrs[1].an_desc->ad_cname.bv_val,0 );
+							entry->e_name.bv_val,cbp->mi->mi_attrs[1].an_desc->ad_cname.bv_val );
 			return 0;
 		}
 		ethers = a->a_vals;
@@ -129,7 +129,7 @@ NSSOV_HANDLE(
 	READ_STRING(fp,cbp.buf);
 	cbp.name.bv_len = tmpint32;
 	cbp.name.bv_val = cbp.buf;,
-	Debug(LDAP_DEBUG_TRACE,"nssov_ether_byname(%s)\n",cbp.name.bv_val,0,0);,
+	Debug(LDAP_DEBUG_TRACE,"nssov_ether_byname(%s)\n",cbp.name.bv_val);,
 	NSLCD_ACTION_ETHER_BYNAME,
 	nssov_filter_byname(cbp.mi,0,&cbp.name,&filter)
 )
@@ -150,7 +150,7 @@ NSSOV_HANDLE(
 		addr.ether_addr_octet[4],
 		addr.ether_addr_octet[5]);
 	cbp.addr.bv_val = cbp.buf;,
-	Debug(LDAP_DEBUG_TRACE,"nssov_ether_byether(%s)\n",cbp.addr.bv_val,0,0);,
+	Debug(LDAP_DEBUG_TRACE,"nssov_ether_byether(%s)\n",cbp.addr.bv_val);,
 	NSLCD_ACTION_ETHER_BYETHER,
 	nssov_filter_byid(cbp.mi,1,&cbp.addr,&filter)
 )
@@ -161,7 +161,7 @@ NSSOV_HANDLE(
 	/* no parameters to read */
 	BER_BVZERO(&cbp.name);
 	BER_BVZERO(&cbp.addr);,
-	Debug(LDAP_DEBUG_TRACE,"nssov_ether_all()\n",0,0,0);,
+	Debug(LDAP_DEBUG_TRACE,"nssov_ether_all()\n");,
 	NSLCD_ACTION_ETHER_ALL,
 	(filter=cbp.mi->mi_filter,0)
 )
