@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2020 The OpenLDAP Foundation.
+ * Copyright 1998-2022 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -1700,7 +1700,7 @@ ure_compile(ucs2_t *re, unsigned long relen, int casefold, ure_buffer_t buf)
 
     /*
      * Reset the various fields of the compilation buffer.  Default the flags
-     * to indicate the presense of the "^$" pattern.  If any other pattern
+     * to indicate the presence of the "^$" pattern.  If any other pattern
      * occurs, then this flag will be removed.  This is done to catch this
      * special pattern and handle it specially when matching.
      */
@@ -1720,7 +1720,7 @@ ure_compile(ucs2_t *re, unsigned long relen, int casefold, ure_buffer_t buf)
     buf->states.states_used = 0;
 
     /*
-     * Construct the NFA.  If this stage returns a 0, then an error occured or
+     * Construct the NFA.  If this stage returns a 0, then an error occurred or
      * an empty expression was passed.
      */
     if ((state = _ure_re2nfa(re, relen, buf)) == _URE_NOOP)
@@ -1859,7 +1859,7 @@ ure_write_dfa(ure_dfa_t dfa, FILE *out)
                     if (sym->props & (1 << k)) {
                         if (h != 0)
                           putc(',', out);
-                        fprintf(out, "%hd", k + 1);
+                        fprintf(out, "%d", k + 1);
                         h = 1;
                     }
                 }
