@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2020 The OpenLDAP Foundation.
+ * Copyright 2001-2022 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -64,7 +64,7 @@ monitor_subsys_conn_init(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_conn_init: "
 			"unable to get entry \"%s\"\n",
-			ms->mss_ndn.bv_val, 0, 0 );
+			ms->mss_ndn.bv_val );
 		return( -1 );
 	}
 
@@ -83,7 +83,7 @@ monitor_subsys_conn_init(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_conn_init: "
 			"unable to create entry \"%s,%s\"\n",
-			bv.bv_val, ms->mss_ndn.bv_val, 0 );
+			bv.bv_val, ms->mss_ndn.bv_val );
 		return( -1 );
 	}
 
@@ -110,7 +110,7 @@ monitor_subsys_conn_init(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_conn_init: "
 			"unable to add entry \"cn=Total,%s\"\n",
-			ms->mss_ndn.bv_val, 0, 0 );
+			ms->mss_ndn.bv_val );
 		return( -1 );
 	}
 
@@ -128,7 +128,7 @@ monitor_subsys_conn_init(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_conn_init: "
 			"unable to create entry \"cn=Total,%s\"\n",
-			ms->mss_ndn.bv_val, 0, 0 );
+			ms->mss_ndn.bv_val );
 		return( -1 );
 	}
 	
@@ -149,7 +149,7 @@ monitor_subsys_conn_init(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_conn_init: "
 			"unable to add entry \"cn=Total,%s\"\n",
-			ms->mss_ndn.bv_val, 0, 0 );
+			ms->mss_ndn.bv_val );
 		return( -1 );
 	}
 
@@ -167,7 +167,7 @@ monitor_subsys_conn_init(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_conn_init: "
 			"unable to create entry \"cn=Current,%s\"\n",
-			ms->mss_ndn.bv_val, 0, 0 );
+			ms->mss_ndn.bv_val );
 		return( -1 );
 	}
 	
@@ -188,7 +188,7 @@ monitor_subsys_conn_init(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_conn_init: "
 			"unable to add entry \"cn=Current,%s\"\n",
-			ms->mss_ndn.bv_val, 0, 0 );
+			ms->mss_ndn.bv_val );
 		return( -1 );
 	}
 	
@@ -219,7 +219,7 @@ monitor_subsys_conn_update(
 	dnRdn( &e->e_nname, &rdn );
 	
 	if ( dn_match( &rdn, &total_bv ) ) {
-		n = connections_nextid();
+		n = connections_nextid() - SLAPD_SYNC_SYNCCONN_OFFSET;
 
 	} else if ( dn_match( &rdn, &current_bv ) ) {
 		Connection	*c;
@@ -229,7 +229,9 @@ monitor_subsys_conn_update(
 				c != NULL;
 				n++, c = connection_next( c, &connindex ) )
 		{
-			/* No Op */ ;
+			/* Ignore outbound connections */
+			if ( c->c_conn_state == SLAP_C_CLIENT )
+				n--;
 		}
 		connection_done( c );
 	}
@@ -300,7 +302,7 @@ conn_create(
 			"unable to create entry "
 			"\"cn=Connection %ld,%s\"\n",
 			c->c_connid, 
-			ms->mss_dn.bv_val, 0 );
+			ms->mss_dn.bv_val );
 		return( -1 );
 	}
 
@@ -367,6 +369,9 @@ conn_create(
 	bv.bv_len = snprintf( buf, sizeof( buf ), "%ld", c->c_n_ops_completed );
 	attr_merge_one( e, mi->mi_ad_monitorConnectionOpsCompleted, &bv, NULL );
 
+	bv.bv_len = snprintf( buf, sizeof( buf ), "%ld", c->c_n_ops_async );
+	attr_merge_one( e, mi->mi_ad_monitorConnectionOpsAsync, &bv, NULL );
+
 	bv.bv_len = snprintf( buf, sizeof( buf ), "%ld", c->c_n_get );
 	attr_merge_one( e, mi->mi_ad_monitorConnectionGet, &bv, NULL );
 
@@ -456,6 +461,10 @@ monitor_subsys_conn_create(
 		{
 			monitor_entry_t 	*mp;
 
+			/* ignore outbound for now, nothing to show */
+			if ( c->c_conn_state == SLAP_C_CLIENT )
+				continue;
+
 			if ( conn_create( mi, c, &e, ms ) != SLAP_CB_CONTINUE
 					|| e == NULL )
 			{
