@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2018 The OpenLDAP Foundation.
+ * Copyright 2001-2022 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -29,7 +29,6 @@
 
 #include <ldap_rq.h>
 
-#ifndef NO_THREADS
 typedef enum {
 	MT_UNKNOWN,
 	MT_RUNQUEUE,
@@ -96,7 +95,6 @@ monitor_subsys_thread_update(
 	Operation		*op,
 	SlapReply		*rs,
 	Entry 			*e );
-#endif /* ! NO_THREADS */
 
 /*
  * initializes log subentry
@@ -106,7 +104,6 @@ monitor_subsys_thread_init(
 	BackendDB       	*be,
 	monitor_subsys_t	*ms )
 {
-#ifndef NO_THREADS
 	monitor_info_t	*mi;
 	monitor_entry_t	*mp;
 	Entry		*e, **ep, *e_thread;
@@ -119,8 +116,7 @@ monitor_subsys_thread_init(
 	if ( monitor_cache_get( mi, &ms->mss_ndn, &e_thread ) ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_thread_init: unable to get entry \"%s\"\n",
-			ms->mss_dn.bv_val, 
-			0, 0 );
+			ms->mss_dn.bv_val );
 		return( -1 );
 	}
 
@@ -145,7 +141,7 @@ monitor_subsys_thread_init(
 				"monitor_subsys_thread_init: "
 				"unable to create entry \"%s,%s\"\n",
 				mt[ i ].rdn.bv_val,
-				ms->mss_ndn.bv_val, 0 );
+				ms->mss_ndn.bv_val );
 			return( -1 );
 		}
 
@@ -201,7 +197,7 @@ monitor_subsys_thread_init(
 				"monitor_subsys_thread_init: "
 				"unable to add entry \"%s,%s\"\n",
 				mt[ i ].rdn.bv_val,
-				ms->mss_dn.bv_val, 0 );
+				ms->mss_dn.bv_val );
 			return( -1 );
 		}
 	
@@ -211,11 +207,9 @@ monitor_subsys_thread_init(
 
 	monitor_cache_release( mi, e_thread );
 
-#endif /* ! NO_THREADS */
 	return( 0 );
 }
 
-#ifndef NO_THREADS
 static int 
 monitor_subsys_thread_update( 
 	Operation		*op,
@@ -355,4 +349,3 @@ monitor_subsys_thread_update(
 
 	return SLAP_CB_CONTINUE;
 }
-#endif /* ! NO_THREADS */
