@@ -2283,6 +2283,8 @@ accesslog_db_destroy(
 		ch_free( li->li_sids );
 	if ( li->li_mincsn )
 		ber_bvarray_free( li->li_mincsn );
+	if ( li->li_db_suffix.bv_val )
+		ch_free( li->li_db_suffix.bv_val );
 	ldap_pvt_thread_mutex_destroy( &li->li_log_mutex );
 	ldap_pvt_thread_mutex_destroy( &li->li_op_rmutex );
 	free( li );
@@ -2666,7 +2668,7 @@ int accesslog_initialize()
 {
 	int i, rc;
 	Syntax *rdnTimestampSyntax;
-	MatchingRule *rdnTimestampMatch;
+	MatchingRule *rdnTimestampMatch, *rdnTimestampOrdering;
 
 	accesslog.on_bi.bi_type = "accesslog";
 	accesslog.on_bi.bi_db_init = accesslog_db_init;
@@ -2730,21 +2732,26 @@ int accesslog_initialize()
 
 	/* Inject custom normalizer for reqStart/reqEnd */
 	rdnTimestampMatch = ch_malloc( sizeof( MatchingRule ));
+	rdnTimestampOrdering = ch_malloc( sizeof( MatchingRule ));
 	rdnTimestampSyntax = ch_malloc( sizeof( Syntax ));
 	*rdnTimestampMatch = *ad_reqStart->ad_type->sat_equality;
 	rdnTimestampMatch->smr_normalize = rdnTimestampNormalize;
+	*rdnTimestampOrdering = *ad_reqStart->ad_type->sat_ordering;
+	rdnTimestampOrdering->smr_normalize = rdnTimestampNormalize;
 	*rdnTimestampSyntax = *ad_reqStart->ad_type->sat_syntax;
 	rdnTimestampSyntax->ssyn_validate = rdnTimestampValidate;
 	ad_reqStart->ad_type->sat_equality = rdnTimestampMatch;
+	ad_reqStart->ad_type->sat_ordering = rdnTimestampOrdering;
 	ad_reqStart->ad_type->sat_syntax = rdnTimestampSyntax;
 
 	rdnTimestampMatch = ch_malloc( sizeof( MatchingRule ));
+	rdnTimestampOrdering = ch_malloc( sizeof( MatchingRule ));
 	rdnTimestampSyntax = ch_malloc( sizeof( Syntax ));
 	*rdnTimestampMatch = *ad_reqStart->ad_type->sat_equality;
-	rdnTimestampMatch->smr_normalize = rdnTimestampNormalize;
+	*rdnTimestampOrdering = *ad_reqStart->ad_type->sat_ordering;
 	*rdnTimestampSyntax = *ad_reqStart->ad_type->sat_syntax;
-	rdnTimestampSyntax->ssyn_validate = rdnTimestampValidate;
 	ad_reqEnd->ad_type->sat_equality = rdnTimestampMatch;
+	ad_reqEnd->ad_type->sat_ordering = rdnTimestampOrdering;
 	ad_reqEnd->ad_type->sat_syntax = rdnTimestampSyntax;
 
 	for ( i=0; locs[i].ot; i++ ) {
