@@ -331,6 +331,9 @@ LDAP_SLAPD_F (void) ava_free LDAP_P((
 	Operation *op,
 	AttributeAssertion *ava,
 	int freeit ));
+LDAP_SLAPD_F (AttributeAssertion *) ava_dup LDAP_P((
+	AttributeAssertion *ava,
+	void *memctx ));
 
 /*
  * backend.c
@@ -800,6 +803,7 @@ LDAP_SLAPD_F (Connection *) connection_init LDAP_P((
 	struct berval *id
 	LDAP_PF_LOCAL_SENDMSG_ARG(struct berval *peerbv)));
 
+LDAP_SLAPD_F (void) operation_counter_init LDAP_P(( Operation *op, void *threadctx ));
 LDAP_SLAPD_F (void) connection_closing LDAP_P((
 	Connection *c, const char *why ));
 LDAP_SLAPD_F (int) connection_is_active LDAP_P(( ber_socket_t s ));
